/*
	sidebar.js
	
	This file contains functions for:
		* manipulating xul for the pagesticky GUI
		* interacting with the RDF datastore
		
*/

//==========================================================
//		RDF Datasource Methods
//----------------------------------------------------------

// These variables will be used to create XPCOM components
var Cc = Components.classes;
var Ci = Components.interfaces;

//var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);

var rdfsession;


/** 
	RDFsession loads the rdf file from the pref directory.
	It is used throughout to access the RDF datasource.
	
	This code was taken from RADwM listing 16.9
*/

function RDFsession() {};
RDFsession.prototype = {
	source_file : "pagesticky.rdf",
	datasource : null,
	
	init:function () {
	try {
		var filedir, file, convert, url, rdf;

		with (window) {
			filedir = Cc["@mozilla.org/file/directory_service;1"];
			filedir = filedir.getService(Ci.nsIProperties);
			
			convert = Cc["@mozilla.org/network/protocol;1?name=file"]
			convert = convert.createInstance(Ci.nsIFileProtocolHandler);
			
			rdf = Cc["@mozilla.org/rdf/rdf-service;1"];
			rdf = rdf.getService(Ci.nsIRDFService);
		}
		
		// get the file from the users profile directory
		file = filedir.get("ProfD", Ci.nsIFile);
		file.append(this.source_file);
		
		// error checking
		if (!file.exists()) {
			alert('This is your first run of pagesticky.  Creating a RDF file in your preferences folder...');
			try {
				file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
				return false;
			}catch(e) {
				alert('didnt make new file: '+e);
			}
		}
		
		if (!file.isFile() || !file.isWritable() || !file.isReadable())
			alert ("Cannot access " + this.source_file);
		
		// convert the file to RDF stream
		url = convert.newFileURI(file);
		this.datasource = rdf.GetDataSourceBlocking(url.spec);
		
		//
		//alert('rdfsession finished loading '+this.source_file);
		return true;
	} catch (e) {
		alert('rdfsession did not init: '+e);
	}
	}
};

function rdf_firstrun () {
	
	var ns = "http://pagesticky.mozdev.org/PS#";
	
	var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
	
	//XPCOM object for generic rdf container
	var container = Components.classes["@mozilla.org/rdf/container;1"];
		container = container.getService(Ci.nsIRDFContainer);
		
	var containerUtils = Cc["@mozilla.org/rdf/container-utils;1"].createInstance(Ci.nsIRDFContainerUtils);
	
	var seqRes = rdf.GetResource("urn:ps:notes");
	containerUtils.MakeSeq(rdfsession.datasource, seqRes);
	
	var flush = rdfsession.datasource;
	flush.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
	flush.Flush();
	
	
	//create blank prefs
	var subject = rdf.GetResource('urn:ps:prefs');
	var predicate = rdf.GetResource("http://pagesticky.mozdev.org/ps#userName");
	var value = rdf.GetLiteral("Welcome, User");
	
	rdfsession.datasource.Assert(subject, predicate, value, true);
	
	var subject = rdf.GetResource('urn:ps:prefs');
	var predicate = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteColor");
	var value = rdf.GetLiteral("notebox_yellow");
	
	rdfsession.datasource.Assert(subject, predicate, value, true);
	
	flushrdf();
	
	//alert('finished rdf file init');
}


//=========================================



var noteidToEdit;
function editInProgress() {
	noteidToEdit = window.document.getElementById('noteList').currentItem.id;
}

function editDone() {
	if (noteidToEdit=="")
		return;
	
	var newTitleString = window.document.getElementById(noteidToEdit).firstChild.childNodes.item(0).value;
	var newTextString = window.document.getElementById(noteidToEdit).firstChild.childNodes.item(1).value;

	var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);

	newTitle = rdf.GetLiteral(newTitleString);
	newText = rdf.GetLiteral(newTextString);
	
	var noteToSave = rdf.GetResource(noteidToEdit);
	var titleProp = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteTitle");
	var textProp = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteText");
	
	var oldTitle = rdfsession.datasource.GetTarget(noteToSave, titleProp, true);
	var oldText = rdfsession.datasource.GetTarget(noteToSave, textProp, true);
	
	rdfsession.datasource.Change(noteToSave, titleProp, oldTitle, newTitle);
	rdfsession.datasource.Change(noteToSave, textProp, oldText, newText);
	
	var flush = rdfsession.datasource;
	flush.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
	flush.Flush();
	
	noteidToEdit = "";
}

function newNote() {

	var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
	var container = Components.classes["@mozilla.org/rdf/container;1"].getService(Ci.nsIRDFContainer);
	var containerUtils = Cc["@mozilla.org/rdf/container-utils;1"].createInstance(Ci.nsIRDFContainerUtils);
	
	var noteBag = rdf.GetResource("urn:ps:notes");
	var pageBag = rdf.GetResource(getCurrentURL());
	var websiteBag = rdf.GetResource(getWebsiteURL());
	
	// add note to noteBag
	container.Init(rdfsession.datasource, noteBag);
	var noteid = "urn:ps:notes:"+generateID();
	newNoteRes = rdf.GetResource(noteid);
	container.AppendElement(newNoteRes);
	
	// add note to pageBag
	try {
		container.Init(rdfsession.datasource, pageBag);
	}
	catch (e) {
		container = containerUtils.MakeSeq(rdfsession.datasource, pageBag);
		
		var flush = rdfsession.datasource;
		flush.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
		flush.Flush();
		rdfsession.init();
	}
	container.AppendElement (newNoteRes);
	
	// add page to website bag??
	
	// put in blank data
	try {
		var subject = rdf.GetResource(noteid);
		var predicate = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteTitle");
		var value = rdf.GetLiteral("New Note");
		
		rdfsession.datasource.Assert(subject, predicate, value, true);
		
		var subject = rdf.GetResource(noteid);
		var predicate = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteText");
		var value = rdf.GetLiteral("Type your notes here");
		
		rdfsession.datasource.Assert(subject, predicate, value, true);
		
	} catch (e) {
		alert('doSave: when creating a new node: '+e);
	}
	
	flushrdf();	
	rebuild();
}

function deleteNote() {

	var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
	var container = Components.classes["@mozilla.org/rdf/container;1"].getService(Ci.nsIRDFContainer);
	var containerUtils = Cc["@mozilla.org/rdf/container-utils;1"].createInstance(Ci.nsIRDFContainerUtils);
	
	var deleteme = rdf.GetResource(noteidToEdit = window.document.getElementById('noteList').currentItem.id);
	
	var noteBag = rdf.GetResource("urn:ps:notes");
	var pageBag = rdf.GetResource(getCurrentURL());
	
	container.Init(rdfsession.datasource, noteBag);
	container.RemoveElement(deleteme, true);
	
	container.Init(rdfsession.datasource, pageBag);
	container.RemoveElement(deleteme, true);
	
	var titleProp = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteTitle");
	var textProp = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteText");
	
	var oldTitle = rdfsession.datasource.GetTarget(deleteme, titleProp, true);
	var oldText = rdfsession.datasource.GetTarget(deleteme, textProp, true);
	
	rdfsession.datasource.Unassert(deleteme, titleProp, oldTitle);
	rdfsession.datasource.Unassert(deleteme, textProp, oldText);
	
	flushrdf();
	rebuild();

}




//=============================================================

function sidebarStartup () {
	// set the datasource for the noteList
	try {
		rdfsession = new RDFsession();
		if (!rdfsession.init()) {
			//alert('startup: rdfsession init failed');
			rdfsession.init();
			rdf_firstrun();
		}
		var noteList = window.document.getElementById('noteList');
		
		noteList.database.AddDataSource(rdfsession.datasource);
		noteList.builder.rebuild();
	} catch (e) {
		alert("Sidebar startup error: "+e);
	}
	//poll url bar every second
	setInterval("pollURL()", 1000);
}

var currentURL;
function pollURL () {
	if (currentURL != getCurrentURL()) {
		currentURL = getCurrentURL();
		showNotes(currentURL);
	}
}


function flushrdf() {
	var flush = rdfsession.datasource;
	flush.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
	flush.Flush();
	rdfsession.init();
}

function rebuild() {
	var noteList = window.document.getElementById('noteList');
	noteList.database.AddDataSource(rdfsession.datasource);
	noteList.builder.rebuild();
	showNotes(currentURL);
}

function getCurrentURL()
{
	//taken from RADwM 14.27
	doc = window.content.document;
	return(doc.location.href);
}

function showNotes (url) {
	//alert('showing notes for: '+url);
	var noteList = window.document.getElementById('noteList');
	noteList.setAttribute('ref', url);
	noteList.builder.rebuild();
}

function getWebsiteURL() {
	var url = getCurrentURL();
	var ws = url.replace(/.com.*/, ".com/");
	return ws;
}

function generateID() {
	randid = Math.floor(Math.random()*813800293);
	return randid;
}

//=============================================================
// prefs

function showPrefs() {
	window.open('chrome://pagesticky/content/pagestickypref.xul', 'pagestickypref','chrome,width=400,height=150');
}


function loadPrefs() {
	//alert('loading prefs');
try {
	var rdfsession = new RDFsession();
	rdfsession.init();

	var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
	
	var prefsObj = rdf.GetResource('urn:ps:prefs');
	var userProp = rdf.GetResource("http://pagesticky.mozdev.org/ps#userName");
	var colorProp = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteColor");
	
	var curUserName = rdfsession.datasource.GetTarget(prefsObj, userProp, true);
	var curUserName = curUserName.QueryInterface(Ci.nsIRDFLiteral);
	
	var curNoteColor = rdfsession.datasource.GetTarget(prefsObj, colorProp, true);
	var curNoteColor = curNoteColor.QueryInterface(Ci.nsIRDFLiteral);
	
	window.document.getElementById('txtUserName').value = curUserName.Value;
	window.document.getElementById('lstNoteColor').value = curNoteColor.Value;
}catch(e) {
	alert(e);
}
	//alert('loaded');
}

function savePrefs() {

try {
	rdfsession = new RDFsession();
	rdfsession.init();
	
	var username = window.document.getElementById('txtUserName').value;
	var color = window.document.getElementById('lstNoteColor').value;

	var rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);

	newName = rdf.GetLiteral(username);
	newColor = rdf.GetLiteral(color);
	
	var prefsObj = rdf.GetResource('urn:ps:prefs');
	var nameProp = rdf.GetResource("http://pagesticky.mozdev.org/ps#userName");
	var colorProp = rdf.GetResource("http://pagesticky.mozdev.org/ps#noteColor");
	
	var oldName = rdfsession.datasource.GetTarget(prefsObj, nameProp, true);
	var oldColor = rdfsession.datasource.GetTarget(prefsObj, colorProp, true);
	
	//if (oldName != newName)
		rdfsession.datasource.Change(prefsObj, nameProp, oldName, newName);
	//if (oldColor != newColor)
		rdfsession.datasource.Change(prefsObj, colorProp, oldColor, newColor);
} catch(e) {
	alert('save prefs err: '+e);
}

	flushrdf();
	window.close();
	alert('You must restart pagesticky panel to enact color changes');
}

